/*
 * Copyright (c) 2008, Witold Wysota
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of Witold Wysota may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <QtGui>
#include <QtCore>

class ThumbRunnable : public QRunnable {
public:
  ThumbRunnable(const QString &img, const QSize &s) 
  : QRunnable(), m_image(img), m_size(s){}
  void run(){ 
    m_result = QImage(m_image).scaled(m_size, Qt::KeepAspectRatio, Qt::SmoothTransformation); 
  }
  const QImage &result() const{
    return m_result; 
  }
private:
  QString m_image;
  QImage m_result;
  QSize m_size;
};

class ListWidget : public QListWidget {
public:
  ListWidget(QList<ThumbRunnable*> &runs) : QListWidget(), m_runs(runs){
        setViewMode(IconMode);
        setResizeMode(Adjust);
        setMovement(Static);
        foreach(ThumbRunnable *r, m_runs)
                remaining << r;
  }
protected:
  void timerEvent(QTimerEvent *){
    // after some period of time check which results are ready
    foreach(ThumbRunnable *r, remaining){
      QImage res = r->result();
      if(res.isNull()) continue; // not ready
      remaining.remove(r);       // ready, remove from runnables list
      QListWidgetItem *item = new QListWidgetItem(this);
      item->setData(Qt::DecorationRole, QPixmap::fromImage(res));
    }
  }
  QSet<ThumbRunnable*> remaining;
  QList<ThumbRunnable*> &m_runs;
};

int main(int argc, char **argv){
  QApplication app(argc, argv);
  QList<ThumbRunnable *> runnables;
  
  QSize s(300,300);
  QStringList images = QFileDialog::getOpenFileNames(0, QObject::tr("Choose a bunch of images..."));
  if(images.isEmpty())
    return 0;
  foreach(const QString &image, images){
    ThumbRunnable *r = new ThumbRunnable(image, s);
    r->setAutoDelete(false); // don't autodelete after the job is finished
    QThreadPool::globalInstance()->start(r);
    runnables << r;
  }
  ListWidget lw(runnables);
  lw.setGridSize(s+QSize(10,10));
  lw.setUniformItemSizes(true);
  lw.showMaximized();
  lw.startTimer(250);
  return app.exec();
}
