/*
 * Copyright (c) 2008, Witold Wysota
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of Witold Wysota may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <QtGui>
#include <QtCore>

typedef QFutureWatcher<QImage> ImageWatcher;

class ListWidget : public QListWidget {
  Q_OBJECT
public:
  ListWidget() : QListWidget(){
        setViewMode(IconMode);
        setResizeMode(Adjust);
        setMovement(Static);
        setVerticalScrollMode(ScrollPerPixel);
        // show a nice progress bar
        progressBar = new QProgressBar(this);
        progressBar->setGeometry(10,10, 200, progressBar->sizeHint().height());
        progressBar->setFormat("%v of %m (%p%)");
  }
  void setFutureWatcher(ImageWatcher *w){
    if(watcher==w)
      return;
    // disconnects should be here...
    watcher = w;
    if(w){
      connect(w, SIGNAL(resultReadyAt(int)), this, SLOT(readyAt(int)));
      connect(w, SIGNAL(progressRangeChanged(int, int)), progressBar, SLOT(setRange(int, int)));
      connect(w, SIGNAL(progressValueChanged(int)), progressBar, SLOT(setValue(int)));
      connect(w, SIGNAL(finished()), progressBar, SLOT(hide()));
      connect(w, SIGNAL(started()), progressBar, SLOT(show()));
    }
  }
private slots:
  void readyAt(int which){
    QFuture<QImage> f = watcher->future();
    QListWidgetItem *item = new QListWidgetItem(this);
    item->setData(Qt::DecorationRole, QPixmap::fromImage(f.resultAt(which)));
  }
protected:
  ImageWatcher *watcher;
  QProgressBar *progressBar;
};

#include "main.moc"

QImage makeThumb(const QString &img){
  return QImage(img).scaled(QSize(300,300), Qt::KeepAspectRatio, Qt::SmoothTransformation);
}


int main(int argc, char **argv){
  QApplication app(argc, argv);
  QStringList images = QFileDialog::getOpenFileNames(0, QObject::tr("Choose a bunch of images..."));
  if(images.isEmpty())
    return 0;
  ImageWatcher watcher;
  QFuture<QImage> result = QtConcurrent::mapped(images, makeThumb);
  watcher.setFuture(result);
  ListWidget w;
  w.setFutureWatcher(&watcher);
  w.showMaximized();
  app.connect(&app, SIGNAL(aboutToQuit()), &watcher, SLOT(cancel()));
  return app.exec();
}
